from protos import ydb_operation_pb2 as _ydb_operation_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AcquireLockRequest(_message.Message):
    __slots__ = ["operation_params", "partition_id", "path"]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PARTITION_ID_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    partition_id: int
    path: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., path: _Optional[str] = ..., partition_id: _Optional[int] = ...) -> None: ...

class AcquireLockResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class AcquireLockResult(_message.Message):
    __slots__ = ["lock_generation", "node_id"]
    LOCK_GENERATION_FIELD_NUMBER: _ClassVar[int]
    NODE_ID_FIELD_NUMBER: _ClassVar[int]
    lock_generation: int
    node_id: int
    def __init__(self, lock_generation: _Optional[int] = ..., node_id: _Optional[int] = ...) -> None: ...

class AlterVolumeRequest(_message.Message):
    __slots__ = ["alter_partition_count", "operation_params", "path", "storage_config"]
    ALTER_PARTITION_COUNT_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    STORAGE_CONFIG_FIELD_NUMBER: _ClassVar[int]
    alter_partition_count: int
    operation_params: _ydb_operation_pb2.OperationParams
    path: str
    storage_config: StorageConfig
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., path: _Optional[str] = ..., alter_partition_count: _Optional[int] = ..., storage_config: _Optional[_Union[StorageConfig, _Mapping]] = ...) -> None: ...

class AlterVolumeResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class AlterVolumeResult(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class CreateVolumeRequest(_message.Message):
    __slots__ = ["operation_params", "partition_count", "path", "storage_config"]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PARTITION_COUNT_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    STORAGE_CONFIG_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    partition_count: int
    path: str
    storage_config: StorageConfig
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., path: _Optional[str] = ..., partition_count: _Optional[int] = ..., storage_config: _Optional[_Union[StorageConfig, _Mapping]] = ...) -> None: ...

class CreateVolumeResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class CreateVolumeResult(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class DescribeVolumeRequest(_message.Message):
    __slots__ = ["operation_params", "path"]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    path: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., path: _Optional[str] = ...) -> None: ...

class DescribeVolumeResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class DescribeVolumeResult(_message.Message):
    __slots__ = ["partition_count", "path"]
    PARTITION_COUNT_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    partition_count: int
    path: str
    def __init__(self, path: _Optional[str] = ..., partition_count: _Optional[int] = ...) -> None: ...

class DropVolumeRequest(_message.Message):
    __slots__ = ["operation_params", "path"]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    path: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., path: _Optional[str] = ...) -> None: ...

class DropVolumeResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class DropVolumeResult(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ExecuteTransactionRequest(_message.Message):
    __slots__ = ["commands", "lock_generation", "operation_params", "partition_id", "path"]
    class Command(_message.Message):
        __slots__ = ["concat", "copy_range", "delete_range", "rename", "write"]
        class Concat(_message.Message):
            __slots__ = ["input_keys", "keep_inputs", "output_key"]
            INPUT_KEYS_FIELD_NUMBER: _ClassVar[int]
            KEEP_INPUTS_FIELD_NUMBER: _ClassVar[int]
            OUTPUT_KEY_FIELD_NUMBER: _ClassVar[int]
            input_keys: _containers.RepeatedScalarFieldContainer[str]
            keep_inputs: bool
            output_key: str
            def __init__(self, input_keys: _Optional[_Iterable[str]] = ..., output_key: _Optional[str] = ..., keep_inputs: bool = ...) -> None: ...
        class CopyRange(_message.Message):
            __slots__ = ["prefix_to_add", "prefix_to_remove", "range"]
            PREFIX_TO_ADD_FIELD_NUMBER: _ClassVar[int]
            PREFIX_TO_REMOVE_FIELD_NUMBER: _ClassVar[int]
            RANGE_FIELD_NUMBER: _ClassVar[int]
            prefix_to_add: str
            prefix_to_remove: str
            range: KeyRange
            def __init__(self, range: _Optional[_Union[KeyRange, _Mapping]] = ..., prefix_to_remove: _Optional[str] = ..., prefix_to_add: _Optional[str] = ...) -> None: ...
        class DeleteRange(_message.Message):
            __slots__ = ["range"]
            RANGE_FIELD_NUMBER: _ClassVar[int]
            range: KeyRange
            def __init__(self, range: _Optional[_Union[KeyRange, _Mapping]] = ...) -> None: ...
        class Rename(_message.Message):
            __slots__ = ["new_key", "old_key"]
            NEW_KEY_FIELD_NUMBER: _ClassVar[int]
            OLD_KEY_FIELD_NUMBER: _ClassVar[int]
            new_key: str
            old_key: str
            def __init__(self, old_key: _Optional[str] = ..., new_key: _Optional[str] = ...) -> None: ...
        class Write(_message.Message):
            __slots__ = ["key", "priority", "storage_channel", "tactic", "value"]
            class Tactic(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
                __slots__ = []
            KEY_FIELD_NUMBER: _ClassVar[int]
            PRIORITY_FIELD_NUMBER: _ClassVar[int]
            STORAGE_CHANNEL_FIELD_NUMBER: _ClassVar[int]
            TACTIC_FIELD_NUMBER: _ClassVar[int]
            TACTIC_MAX_THROUGHPUT: ExecuteTransactionRequest.Command.Write.Tactic
            TACTIC_MIN_LATENCY: ExecuteTransactionRequest.Command.Write.Tactic
            TACTIC_UNSPECIFIED: ExecuteTransactionRequest.Command.Write.Tactic
            VALUE_FIELD_NUMBER: _ClassVar[int]
            key: str
            priority: Priorities.Priority
            storage_channel: int
            tactic: ExecuteTransactionRequest.Command.Write.Tactic
            value: bytes
            def __init__(self, key: _Optional[str] = ..., value: _Optional[bytes] = ..., storage_channel: _Optional[int] = ..., priority: _Optional[_Union[Priorities.Priority, str]] = ..., tactic: _Optional[_Union[ExecuteTransactionRequest.Command.Write.Tactic, str]] = ...) -> None: ...
        CONCAT_FIELD_NUMBER: _ClassVar[int]
        COPY_RANGE_FIELD_NUMBER: _ClassVar[int]
        DELETE_RANGE_FIELD_NUMBER: _ClassVar[int]
        RENAME_FIELD_NUMBER: _ClassVar[int]
        WRITE_FIELD_NUMBER: _ClassVar[int]
        concat: ExecuteTransactionRequest.Command.Concat
        copy_range: ExecuteTransactionRequest.Command.CopyRange
        delete_range: ExecuteTransactionRequest.Command.DeleteRange
        rename: ExecuteTransactionRequest.Command.Rename
        write: ExecuteTransactionRequest.Command.Write
        def __init__(self, delete_range: _Optional[_Union[ExecuteTransactionRequest.Command.DeleteRange, _Mapping]] = ..., rename: _Optional[_Union[ExecuteTransactionRequest.Command.Rename, _Mapping]] = ..., copy_range: _Optional[_Union[ExecuteTransactionRequest.Command.CopyRange, _Mapping]] = ..., concat: _Optional[_Union[ExecuteTransactionRequest.Command.Concat, _Mapping]] = ..., write: _Optional[_Union[ExecuteTransactionRequest.Command.Write, _Mapping]] = ...) -> None: ...
    COMMANDS_FIELD_NUMBER: _ClassVar[int]
    LOCK_GENERATION_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PARTITION_ID_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    commands: _containers.RepeatedCompositeFieldContainer[ExecuteTransactionRequest.Command]
    lock_generation: int
    operation_params: _ydb_operation_pb2.OperationParams
    partition_id: int
    path: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., path: _Optional[str] = ..., partition_id: _Optional[int] = ..., lock_generation: _Optional[int] = ..., commands: _Optional[_Iterable[_Union[ExecuteTransactionRequest.Command, _Mapping]]] = ...) -> None: ...

class ExecuteTransactionResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ExecuteTransactionResult(_message.Message):
    __slots__ = ["node_id", "storage_channel_info"]
    NODE_ID_FIELD_NUMBER: _ClassVar[int]
    STORAGE_CHANNEL_INFO_FIELD_NUMBER: _ClassVar[int]
    node_id: int
    storage_channel_info: _containers.RepeatedCompositeFieldContainer[StorageChannelInfo]
    def __init__(self, storage_channel_info: _Optional[_Iterable[_Union[StorageChannelInfo, _Mapping]]] = ..., node_id: _Optional[int] = ...) -> None: ...

class GetStorageChannelStatusRequest(_message.Message):
    __slots__ = ["lock_generation", "operation_params", "partition_id", "path", "storage_channel"]
    LOCK_GENERATION_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PARTITION_ID_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    STORAGE_CHANNEL_FIELD_NUMBER: _ClassVar[int]
    lock_generation: int
    operation_params: _ydb_operation_pb2.OperationParams
    partition_id: int
    path: str
    storage_channel: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., path: _Optional[str] = ..., partition_id: _Optional[int] = ..., lock_generation: _Optional[int] = ..., storage_channel: _Optional[_Iterable[int]] = ...) -> None: ...

class GetStorageChannelStatusResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class GetStorageChannelStatusResult(_message.Message):
    __slots__ = ["node_id", "storage_channel_info"]
    NODE_ID_FIELD_NUMBER: _ClassVar[int]
    STORAGE_CHANNEL_INFO_FIELD_NUMBER: _ClassVar[int]
    node_id: int
    storage_channel_info: _containers.RepeatedCompositeFieldContainer[StorageChannelInfo]
    def __init__(self, storage_channel_info: _Optional[_Iterable[_Union[StorageChannelInfo, _Mapping]]] = ..., node_id: _Optional[int] = ...) -> None: ...

class KeyRange(_message.Message):
    __slots__ = ["from_key_exclusive", "from_key_inclusive", "to_key_exclusive", "to_key_inclusive"]
    FROM_KEY_EXCLUSIVE_FIELD_NUMBER: _ClassVar[int]
    FROM_KEY_INCLUSIVE_FIELD_NUMBER: _ClassVar[int]
    TO_KEY_EXCLUSIVE_FIELD_NUMBER: _ClassVar[int]
    TO_KEY_INCLUSIVE_FIELD_NUMBER: _ClassVar[int]
    from_key_exclusive: str
    from_key_inclusive: str
    to_key_exclusive: str
    to_key_inclusive: str
    def __init__(self, from_key_inclusive: _Optional[str] = ..., from_key_exclusive: _Optional[str] = ..., to_key_inclusive: _Optional[str] = ..., to_key_exclusive: _Optional[str] = ...) -> None: ...

class ListLocalPartitionsRequest(_message.Message):
    __slots__ = ["node_id", "operation_params", "path"]
    NODE_ID_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    node_id: int
    operation_params: _ydb_operation_pb2.OperationParams
    path: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., path: _Optional[str] = ..., node_id: _Optional[int] = ...) -> None: ...

class ListLocalPartitionsResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ListLocalPartitionsResult(_message.Message):
    __slots__ = ["node_id", "partition_ids", "path"]
    NODE_ID_FIELD_NUMBER: _ClassVar[int]
    PARTITION_IDS_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    node_id: int
    partition_ids: _containers.RepeatedScalarFieldContainer[int]
    path: str
    def __init__(self, path: _Optional[str] = ..., node_id: _Optional[int] = ..., partition_ids: _Optional[_Iterable[int]] = ...) -> None: ...

class ListRangeRequest(_message.Message):
    __slots__ = ["limit_bytes", "lock_generation", "operation_params", "partition_id", "path", "range"]
    LIMIT_BYTES_FIELD_NUMBER: _ClassVar[int]
    LOCK_GENERATION_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PARTITION_ID_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    RANGE_FIELD_NUMBER: _ClassVar[int]
    limit_bytes: int
    lock_generation: int
    operation_params: _ydb_operation_pb2.OperationParams
    partition_id: int
    path: str
    range: KeyRange
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., path: _Optional[str] = ..., partition_id: _Optional[int] = ..., lock_generation: _Optional[int] = ..., range: _Optional[_Union[KeyRange, _Mapping]] = ..., limit_bytes: _Optional[int] = ...) -> None: ...

class ListRangeResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ListRangeResult(_message.Message):
    __slots__ = ["is_overrun", "key", "node_id"]
    class KeyInfo(_message.Message):
        __slots__ = ["creation_unix_time", "key", "storage_channel", "value_size"]
        CREATION_UNIX_TIME_FIELD_NUMBER: _ClassVar[int]
        KEY_FIELD_NUMBER: _ClassVar[int]
        STORAGE_CHANNEL_FIELD_NUMBER: _ClassVar[int]
        VALUE_SIZE_FIELD_NUMBER: _ClassVar[int]
        creation_unix_time: int
        key: str
        storage_channel: int
        value_size: int
        def __init__(self, key: _Optional[str] = ..., value_size: _Optional[int] = ..., creation_unix_time: _Optional[int] = ..., storage_channel: _Optional[int] = ...) -> None: ...
    IS_OVERRUN_FIELD_NUMBER: _ClassVar[int]
    KEY_FIELD_NUMBER: _ClassVar[int]
    NODE_ID_FIELD_NUMBER: _ClassVar[int]
    is_overrun: bool
    key: _containers.RepeatedCompositeFieldContainer[ListRangeResult.KeyInfo]
    node_id: int
    def __init__(self, key: _Optional[_Iterable[_Union[ListRangeResult.KeyInfo, _Mapping]]] = ..., is_overrun: bool = ..., node_id: _Optional[int] = ...) -> None: ...

class Priorities(_message.Message):
    __slots__ = []
    class Priority(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    PRIORITY_BACKGROUND: Priorities.Priority
    PRIORITY_REALTIME: Priorities.Priority
    PRIORITY_UNSPECIFIED: Priorities.Priority
    def __init__(self) -> None: ...

class ReadRangeRequest(_message.Message):
    __slots__ = ["limit_bytes", "lock_generation", "operation_params", "partition_id", "path", "priority", "range"]
    LIMIT_BYTES_FIELD_NUMBER: _ClassVar[int]
    LOCK_GENERATION_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PARTITION_ID_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_FIELD_NUMBER: _ClassVar[int]
    RANGE_FIELD_NUMBER: _ClassVar[int]
    limit_bytes: int
    lock_generation: int
    operation_params: _ydb_operation_pb2.OperationParams
    partition_id: int
    path: str
    priority: Priorities.Priority
    range: KeyRange
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., path: _Optional[str] = ..., partition_id: _Optional[int] = ..., lock_generation: _Optional[int] = ..., range: _Optional[_Union[KeyRange, _Mapping]] = ..., limit_bytes: _Optional[int] = ..., priority: _Optional[_Union[Priorities.Priority, str]] = ...) -> None: ...

class ReadRangeResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ReadRangeResult(_message.Message):
    __slots__ = ["is_overrun", "node_id", "pair"]
    class KeyValuePair(_message.Message):
        __slots__ = ["creation_unix_time", "key", "storage_channel", "value"]
        CREATION_UNIX_TIME_FIELD_NUMBER: _ClassVar[int]
        KEY_FIELD_NUMBER: _ClassVar[int]
        STORAGE_CHANNEL_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        creation_unix_time: int
        key: str
        storage_channel: int
        value: bytes
        def __init__(self, key: _Optional[str] = ..., value: _Optional[bytes] = ..., creation_unix_time: _Optional[int] = ..., storage_channel: _Optional[int] = ...) -> None: ...
    IS_OVERRUN_FIELD_NUMBER: _ClassVar[int]
    NODE_ID_FIELD_NUMBER: _ClassVar[int]
    PAIR_FIELD_NUMBER: _ClassVar[int]
    is_overrun: bool
    node_id: int
    pair: _containers.RepeatedCompositeFieldContainer[ReadRangeResult.KeyValuePair]
    def __init__(self, pair: _Optional[_Iterable[_Union[ReadRangeResult.KeyValuePair, _Mapping]]] = ..., is_overrun: bool = ..., node_id: _Optional[int] = ...) -> None: ...

class ReadRequest(_message.Message):
    __slots__ = ["key", "limit_bytes", "lock_generation", "offset", "operation_params", "partition_id", "path", "priority", "size"]
    KEY_FIELD_NUMBER: _ClassVar[int]
    LIMIT_BYTES_FIELD_NUMBER: _ClassVar[int]
    LOCK_GENERATION_FIELD_NUMBER: _ClassVar[int]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PARTITION_ID_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_FIELD_NUMBER: _ClassVar[int]
    SIZE_FIELD_NUMBER: _ClassVar[int]
    key: str
    limit_bytes: int
    lock_generation: int
    offset: int
    operation_params: _ydb_operation_pb2.OperationParams
    partition_id: int
    path: str
    priority: Priorities.Priority
    size: int
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., path: _Optional[str] = ..., partition_id: _Optional[int] = ..., lock_generation: _Optional[int] = ..., key: _Optional[str] = ..., offset: _Optional[int] = ..., size: _Optional[int] = ..., limit_bytes: _Optional[int] = ..., priority: _Optional[_Union[Priorities.Priority, str]] = ...) -> None: ...

class ReadResponse(_message.Message):
    __slots__ = ["operation"]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ReadResult(_message.Message):
    __slots__ = ["is_overrun", "node_id", "requested_key", "requested_offset", "requested_size", "value"]
    IS_OVERRUN_FIELD_NUMBER: _ClassVar[int]
    NODE_ID_FIELD_NUMBER: _ClassVar[int]
    REQUESTED_KEY_FIELD_NUMBER: _ClassVar[int]
    REQUESTED_OFFSET_FIELD_NUMBER: _ClassVar[int]
    REQUESTED_SIZE_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    is_overrun: bool
    node_id: int
    requested_key: str
    requested_offset: int
    requested_size: int
    value: bytes
    def __init__(self, requested_key: _Optional[str] = ..., requested_offset: _Optional[int] = ..., requested_size: _Optional[int] = ..., value: _Optional[bytes] = ..., is_overrun: bool = ..., node_id: _Optional[int] = ...) -> None: ...

class StorageChannelInfo(_message.Message):
    __slots__ = ["status_flag", "storage_channel"]
    class StatusFlag(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    STATUS_FLAG_FIELD_NUMBER: _ClassVar[int]
    STATUS_FLAG_GREEN: StorageChannelInfo.StatusFlag
    STATUS_FLAG_ORANGE_OUT_SPACE: StorageChannelInfo.StatusFlag
    STATUS_FLAG_UNSPECIFIED: StorageChannelInfo.StatusFlag
    STATUS_FLAG_YELLOW_STOP: StorageChannelInfo.StatusFlag
    STORAGE_CHANNEL_FIELD_NUMBER: _ClassVar[int]
    status_flag: StorageChannelInfo.StatusFlag
    storage_channel: int
    def __init__(self, storage_channel: _Optional[int] = ..., status_flag: _Optional[_Union[StorageChannelInfo.StatusFlag, str]] = ...) -> None: ...

class StorageConfig(_message.Message):
    __slots__ = ["channel"]
    class ChannelConfig(_message.Message):
        __slots__ = ["media"]
        MEDIA_FIELD_NUMBER: _ClassVar[int]
        media: str
        def __init__(self, media: _Optional[str] = ...) -> None: ...
    CHANNEL_FIELD_NUMBER: _ClassVar[int]
    channel: _containers.RepeatedCompositeFieldContainer[StorageConfig.ChannelConfig]
    def __init__(self, channel: _Optional[_Iterable[_Union[StorageConfig.ChannelConfig, _Mapping]]] = ...) -> None: ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from ydb._grpc.v6.protos import ydb_scripting_pb2 as protos_dot_ydb__scripting__pb2

GRPC_GENERATED_VERSION = '1.76.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + ' but the generated code in ydb_scripting_v1_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class ScriptingServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ExecuteYql = channel.unary_unary(
                '/Ydb.Scripting.V1.ScriptingService/ExecuteYql',
                request_serializer=protos_dot_ydb__scripting__pb2.ExecuteYqlRequest.SerializeToString,
                response_deserializer=protos_dot_ydb__scripting__pb2.ExecuteYqlResponse.FromString,
                _registered_method=True)
        self.StreamExecuteYql = channel.unary_stream(
                '/Ydb.Scripting.V1.ScriptingService/StreamExecuteYql',
                request_serializer=protos_dot_ydb__scripting__pb2.ExecuteYqlRequest.SerializeToString,
                response_deserializer=protos_dot_ydb__scripting__pb2.ExecuteYqlPartialResponse.FromString,
                _registered_method=True)
        self.ExplainYql = channel.unary_unary(
                '/Ydb.Scripting.V1.ScriptingService/ExplainYql',
                request_serializer=protos_dot_ydb__scripting__pb2.ExplainYqlRequest.SerializeToString,
                response_deserializer=protos_dot_ydb__scripting__pb2.ExplainYqlResponse.FromString,
                _registered_method=True)


class ScriptingServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ExecuteYql(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamExecuteYql(self, request, context):
        """Executes yql request with streaming result.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExplainYql(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ScriptingServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ExecuteYql': grpc.unary_unary_rpc_method_handler(
                    servicer.ExecuteYql,
                    request_deserializer=protos_dot_ydb__scripting__pb2.ExecuteYqlRequest.FromString,
                    response_serializer=protos_dot_ydb__scripting__pb2.ExecuteYqlResponse.SerializeToString,
            ),
            'StreamExecuteYql': grpc.unary_stream_rpc_method_handler(
                    servicer.StreamExecuteYql,
                    request_deserializer=protos_dot_ydb__scripting__pb2.ExecuteYqlRequest.FromString,
                    response_serializer=protos_dot_ydb__scripting__pb2.ExecuteYqlPartialResponse.SerializeToString,
            ),
            'ExplainYql': grpc.unary_unary_rpc_method_handler(
                    servicer.ExplainYql,
                    request_deserializer=protos_dot_ydb__scripting__pb2.ExplainYqlRequest.FromString,
                    response_serializer=protos_dot_ydb__scripting__pb2.ExplainYqlResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'Ydb.Scripting.V1.ScriptingService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('Ydb.Scripting.V1.ScriptingService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class ScriptingService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ExecuteYql(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.Scripting.V1.ScriptingService/ExecuteYql',
            protos_dot_ydb__scripting__pb2.ExecuteYqlRequest.SerializeToString,
            protos_dot_ydb__scripting__pb2.ExecuteYqlResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def StreamExecuteYql(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/Ydb.Scripting.V1.ScriptingService/StreamExecuteYql',
            protos_dot_ydb__scripting__pb2.ExecuteYqlRequest.SerializeToString,
            protos_dot_ydb__scripting__pb2.ExecuteYqlPartialResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ExplainYql(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.Scripting.V1.ScriptingService/ExplainYql',
            protos_dot_ydb__scripting__pb2.ExplainYqlRequest.SerializeToString,
            protos_dot_ydb__scripting__pb2.ExplainYqlResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



import typing
from dataclasses import dataclass

from .ydb_coordination_public_types import NodeConfig

if typing.TYPE_CHECKING:
    from ..v4.protos import ydb_coordination_pb2
else:
    from ..common.protos import ydb_coordination_pb2

from .common_utils import IToProto


@dataclass
class CreateNodeRequest(IToProto):
    path: str
    config: typing.Optional[NodeConfig]

    def to_proto(self) -> "ydb_coordination_pb2.CreateNodeRequest":
        cfg_proto = self.config.to_proto() if self.config else None
        return ydb_coordination_pb2.CreateNodeRequest(
            path=self.path,
            config=cfg_proto,
        )


@dataclass
class AlterNodeRequest(IToProto):
    path: str
    config: NodeConfig

    def to_proto(self) -> "ydb_coordination_pb2.AlterNodeRequest":
        cfg_proto = self.config.to_proto() if self.config else None
        return ydb_coordination_pb2.AlterNodeRequest(
            path=self.path,
            config=cfg_proto,
        )


@dataclass
class DescribeNodeRequest(IToProto):
    path: str

    def to_proto(self) -> "ydb_coordination_pb2.DescribeNodeRequest":
        return ydb_coordination_pb2.DescribeNodeRequest(
            path=self.path,
        )


@dataclass
class DropNodeRequest(IToProto):
    path: str

    def to_proto(self) -> "ydb_coordination_pb2.DropNodeRequest":
        return ydb_coordination_pb2.DropNodeRequest(
            path=self.path,
        )


@dataclass
class SessionStart(IToProto):
    path: str
    timeout_millis: int
    description: str = ""
    session_id: int = 0
    seq_no: int = 0
    protection_key: bytes = b""

    def to_proto(self) -> "ydb_coordination_pb2.SessionRequest":
        return ydb_coordination_pb2.SessionRequest(
            session_start=ydb_coordination_pb2.SessionRequest.SessionStart(
                path=self.path,
                session_id=self.session_id,
                timeout_millis=self.timeout_millis,
                description=self.description,
                seq_no=self.seq_no,
                protection_key=self.protection_key,
            )
        )


@dataclass
class SessionStop(IToProto):
    def to_proto(self) -> "ydb_coordination_pb2.SessionRequest":
        return ydb_coordination_pb2.SessionRequest(session_stop=ydb_coordination_pb2.SessionRequest.SessionStop())


@dataclass
class Ping(IToProto):
    opaque: int = 0

    def to_proto(self) -> "ydb_coordination_pb2.SessionRequest":
        return ydb_coordination_pb2.SessionRequest(
            ping=ydb_coordination_pb2.SessionRequest.PingPong(opaque=self.opaque)
        )


@dataclass
class CreateSemaphore(IToProto):
    name: str
    req_id: int
    limit: int
    data: bytes = b""

    def to_proto(self) -> "ydb_coordination_pb2.SessionRequest":
        return ydb_coordination_pb2.SessionRequest(
            create_semaphore=ydb_coordination_pb2.SessionRequest.CreateSemaphore(
                req_id=self.req_id, name=self.name, limit=self.limit, data=self.data
            )
        )


@dataclass
class AcquireSemaphore(IToProto):
    name: str
    req_id: int
    count: int = 1
    timeout_millis: int = 0
    data: bytes = b""
    ephemeral: bool = False

    def to_proto(self) -> "ydb_coordination_pb2.SessionRequest":
        return ydb_coordination_pb2.SessionRequest(
            acquire_semaphore=ydb_coordination_pb2.SessionRequest.AcquireSemaphore(
                req_id=self.req_id,
                name=self.name,
                timeout_millis=self.timeout_millis,
                count=self.count,
                data=self.data,
                ephemeral=self.ephemeral,
            )
        )


@dataclass
class ReleaseSemaphore(IToProto):
    name: str
    req_id: int

    def to_proto(self) -> "ydb_coordination_pb2.SessionRequest":
        return ydb_coordination_pb2.SessionRequest(
            release_semaphore=ydb_coordination_pb2.SessionRequest.ReleaseSemaphore(req_id=self.req_id, name=self.name)
        )


@dataclass
class DescribeSemaphore(IToProto):
    include_owners: bool
    include_waiters: bool
    name: str
    req_id: int
    watch_data: bool
    watch_owners: bool

    def to_proto(self) -> "ydb_coordination_pb2.SessionRequest":
        return ydb_coordination_pb2.SessionRequest(
            describe_semaphore=ydb_coordination_pb2.SessionRequest.DescribeSemaphore(
                include_owners=self.include_owners,
                include_waiters=self.include_waiters,
                name=self.name,
                req_id=self.req_id,
                watch_data=self.watch_data,
                watch_owners=self.watch_owners,
            )
        )


@dataclass
class UpdateSemaphore(IToProto):
    name: str
    req_id: int
    data: bytes

    def to_proto(self) -> "ydb_coordination_pb2.SessionRequest":
        return ydb_coordination_pb2.SessionRequest(
            update_semaphore=ydb_coordination_pb2.SessionRequest.UpdateSemaphore(
                req_id=self.req_id, name=self.name, data=self.data
            )
        )


@dataclass
class DeleteSemaphore(IToProto):
    name: str
    req_id: int
    force: bool = False

    def to_proto(self) -> "ydb_coordination_pb2.SessionRequest":
        return ydb_coordination_pb2.SessionRequest(
            delete_semaphore=ydb_coordination_pb2.SessionRequest.DeleteSemaphore(
                req_id=self.req_id, name=self.name, force=self.force
            )
        )


@dataclass
class FromServer:
    raw: "ydb_coordination_pb2.SessionResponse"

    @staticmethod
    def from_proto(resp: "ydb_coordination_pb2.SessionResponse") -> "FromServer":
        return FromServer(raw=resp)

    def __getattr__(self, name: str):
        return getattr(self.raw, name)

    @property
    def session_started(self) -> typing.Optional["ydb_coordination_pb2.SessionResponse.SessionStarted"]:
        s = self.raw.session_started
        return s if s.session_id else None

    @property
    def opaque(self) -> typing.Optional[int]:
        if self.raw.HasField("ping"):
            return self.raw.ping.opaque
        return None

    @property
    def acquire_semaphore_result(self):
        return self.raw.acquire_semaphore_result if self.raw.HasField("acquire_semaphore_result") else None

    @property
    def create_semaphore_result(self):
        return self.raw.create_semaphore_result if self.raw.HasField("create_semaphore_result") else None



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb_monitoring_v1.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb._grpc.v3.protos import ydb_monitoring_pb2 as protos_dot_ydb__monitoring__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ydb_monitoring_v1.proto',
  package='Ydb.Monitoring.V1',
  syntax='proto3',
  serialized_options=b'\n\034tech.ydb.proto.monitoring.v1Z9github.com/ydb-platform/ydb-go-genproto/Ydb_Monitoring_V1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x17ydb_monitoring_v1.proto\x12\x11Ydb.Monitoring.V1\x1a\x1bprotos/ydb_monitoring.proto2\xb7\x01\n\x11MonitoringService\x12P\n\tSelfCheck\x12 .Ydb.Monitoring.SelfCheckRequest\x1a!.Ydb.Monitoring.SelfCheckResponse\x12P\n\tNodeCheck\x12 .Ydb.Monitoring.NodeCheckRequest\x1a!.Ydb.Monitoring.NodeCheckResponseBY\n\x1ctech.ydb.proto.monitoring.v1Z9github.com/ydb-platform/ydb-go-genproto/Ydb_Monitoring_V1b\x06proto3'
  ,
  dependencies=[protos_dot_ydb__monitoring__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)


DESCRIPTOR._options = None

_MONITORINGSERVICE = _descriptor.ServiceDescriptor(
  name='MonitoringService',
  full_name='Ydb.Monitoring.V1.MonitoringService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=76,
  serialized_end=259,
  methods=[
  _descriptor.MethodDescriptor(
    name='SelfCheck',
    full_name='Ydb.Monitoring.V1.MonitoringService.SelfCheck',
    index=0,
    containing_service=None,
    input_type=protos_dot_ydb__monitoring__pb2._SELFCHECKREQUEST,
    output_type=protos_dot_ydb__monitoring__pb2._SELFCHECKRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='NodeCheck',
    full_name='Ydb.Monitoring.V1.MonitoringService.NodeCheck',
    index=1,
    containing_service=None,
    input_type=protos_dot_ydb__monitoring__pb2._NODECHECKREQUEST,
    output_type=protos_dot_ydb__monitoring__pb2._NODECHECKRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_MONITORINGSERVICE)

DESCRIPTOR.services_by_name['MonitoringService'] = _MONITORINGSERVICE

# @@protoc_insertion_point(module_scope)



import abc
import logging
import enum
import functools
from typing import (
    Iterable,
    Optional,
    TYPE_CHECKING,
)

from .. import (
    _apis,
    issues,
)
from .._grpc.grpcwrapper import ydb_topic as _ydb_topic
from .._grpc.grpcwrapper import ydb_query as _ydb_query
from ..connection import _RpcState as RpcState

from . import base
from ..settings import BaseRequestSettings

if TYPE_CHECKING:
    from .session import BaseQuerySession

logger = logging.getLogger(__name__)


class QueryTxStateEnum(enum.Enum):
    NOT_INITIALIZED = "NOT_INITIALIZED"
    BEGINED = "BEGINED"
    COMMITTED = "COMMITTED"
    ROLLBACKED = "ROLLBACKED"
    DEAD = "DEAD"


class QueryTxStateHelper(abc.ABC):
    _VALID_TRANSITIONS = {
        QueryTxStateEnum.NOT_INITIALIZED: [
            QueryTxStateEnum.BEGINED,
            QueryTxStateEnum.DEAD,
            QueryTxStateEnum.COMMITTED,
            QueryTxStateEnum.ROLLBACKED,
        ],
        QueryTxStateEnum.BEGINED: [QueryTxStateEnum.COMMITTED, QueryTxStateEnum.ROLLBACKED, QueryTxStateEnum.DEAD],
        QueryTxStateEnum.COMMITTED: [],
        QueryTxStateEnum.ROLLBACKED: [],
        QueryTxStateEnum.DEAD: [],
    }

    _SKIP_TRANSITIONS = {
        QueryTxStateEnum.NOT_INITIALIZED: [],
        QueryTxStateEnum.BEGINED: [],
        QueryTxStateEnum.COMMITTED: [QueryTxStateEnum.COMMITTED, QueryTxStateEnum.ROLLBACKED],
        QueryTxStateEnum.ROLLBACKED: [QueryTxStateEnum.COMMITTED, QueryTxStateEnum.ROLLBACKED],
        QueryTxStateEnum.DEAD: [QueryTxStateEnum.ROLLBACKED],
    }

    @classmethod
    def valid_transition(cls, before: QueryTxStateEnum, after: QueryTxStateEnum) -> bool:
        return after in cls._VALID_TRANSITIONS[before]

    @classmethod
    def should_skip(cls, before: QueryTxStateEnum, after: QueryTxStateEnum) -> bool:
        return after in cls._SKIP_TRANSITIONS[before]

    @classmethod
    def terminal(cls, state: QueryTxStateEnum) -> bool:
        return len(cls._VALID_TRANSITIONS[state]) == 0


def reset_tx_id_handler(func):
    @functools.wraps(func)
    def decorator(rpc_state, response_pb, session: "BaseQuerySession", tx_state: "QueryTxState", *args, **kwargs):
        try:
            return func(rpc_state, response_pb, session, tx_state, *args, **kwargs)
        except issues.Error:
            tx_state._change_state(QueryTxStateEnum.DEAD)
            tx_state.tx_id = None
            raise

    return decorator


class QueryTxState:
    def __init__(self, tx_mode: base.BaseQueryTxMode):
        """
        Holds transaction context manager info
        :param tx_mode: A mode of transaction
        """
        self.tx_id = None
        self.tx_mode = tx_mode
        self._state = QueryTxStateEnum.NOT_INITIALIZED

    def _check_invalid_transition(self, target: QueryTxStateEnum) -> None:
        if not QueryTxStateHelper.valid_transition(self._state, target):
            raise RuntimeError(f"Transaction could not be moved from {self._state.value} to {target.value}")

    def _change_state(self, target: QueryTxStateEnum) -> None:
        self._check_invalid_transition(target)
        self._state = target

    def _check_tx_ready_to_use(self) -> None:
        if QueryTxStateHelper.terminal(self._state):
            raise RuntimeError(f"Transaction is in terminal state: {self._state.value}")

    def _should_skip(self, target: QueryTxStateEnum) -> bool:
        return QueryTxStateHelper.should_skip(self._state, target)


def _construct_tx_settings(tx_state: QueryTxState) -> _ydb_query.TransactionSettings:
    tx_settings = _ydb_query.TransactionSettings.from_public(tx_state.tx_mode)
    return tx_settings


def _create_begin_transaction_request(
    session: "BaseQuerySession", tx_state: QueryTxState
) -> _apis.ydb_query.BeginTransactionRequest:
    request = _ydb_query.BeginTransactionRequest(
        session_id=session.session_id,
        tx_settings=_construct_tx_settings(tx_state),
    ).to_proto()
    return request


def _create_commit_transaction_request(
    session: "BaseQuerySession", tx_state: QueryTxState
) -> _apis.ydb_query.CommitTransactionRequest:
    request = _apis.ydb_query.CommitTransactionRequest()
    request.tx_id = tx_state.tx_id
    request.session_id = session.session_id
    return request


def _create_rollback_transaction_request(
    session: "BaseQuerySession", tx_state: QueryTxState
) -> _apis.ydb_query.RollbackTransactionRequest:
    request = _apis.ydb_query.RollbackTransactionRequest()
    request.tx_id = tx_state.tx_id
    request.session_id = session.session_id
    return request


@base.bad_session_handler
def wrap_tx_begin_response(
    rpc_state: RpcState,
    response_pb: _apis.ydb_query.BeginTransactionResponse,
    session: "BaseQuerySession",
    tx_state: QueryTxState,
    tx: "BaseQueryTxContext",
) -> "BaseQueryTxContext":
    message = _ydb_query.BeginTransactionResponse.from_proto(response_pb)
    issues._process_response(message.status)
    tx_state._change_state(QueryTxStateEnum.BEGINED)
    tx_state.tx_id = message.tx_meta.tx_id
    return tx


@base.bad_session_handler
@reset_tx_id_handler
def wrap_tx_commit_response(
    rpc_state: RpcState,
    response_pb: _apis.ydb_query.CommitTransactionResponse,
    session: "BaseQuerySession",
    tx_state: QueryTxState,
    tx: "BaseQueryTxContext",
) -> "BaseQueryTxContext":
    message = _ydb_query.CommitTransactionResponse.from_proto(response_pb)
    issues._process_response(message.status)
    tx_state._change_state(QueryTxStateEnum.COMMITTED)
    return tx


@base.bad_session_handler
@reset_tx_id_handler
def wrap_tx_rollback_response(
    rpc_state: RpcState,
    response_pb: _apis.ydb_query.RollbackTransactionResponse,
    session: "BaseQuerySession",
    tx_state: QueryTxState,
    tx: "BaseQueryTxContext",
) -> "BaseQueryTxContext":
    message = _ydb_query.RollbackTransactionResponse.from_proto(response_pb)
    issues._process_response(message.status)
    tx_state._change_state(QueryTxStateEnum.ROLLBACKED)
    return tx


class BaseQueryTxContext(base.CallbackHandler):
    def __init__(self, driver, session: "BaseQuerySession", tx_mode: base.BaseQueryTxMode):
        """
        An object that provides a simple transaction context manager that allows statements execution
        in a transaction. You don't have to open transaction explicitly, because context manager encapsulates
        transaction control logic, and opens new transaction if:

        1) By explicit .begin() method;
        2) On execution of a first statement, which is strictly recommended method, because that avoids useless round trip

        This context manager is not thread-safe, so you should not manipulate on it concurrently.

        :param driver: A driver instance
        :param session: A session instance
        :param tx_mode: Transaction mode, which is a one from the following choices:
         1) QuerySerializableReadWrite() which is default mode;
         2) QueryOnlineReadOnly(allow_inconsistent_reads=False);
         3) QuerySnapshotReadOnly();
         4) QuerySnapshotReadWrite();
         5) QueryStaleReadOnly().
        """

        self._driver = driver
        self._tx_state = QueryTxState(tx_mode)
        self.session = session
        self._prev_stream = None
        self._external_error = None
        self._last_query_stats = None

    @property
    def session_id(self) -> str:
        """
        A transaction's session id

        :return: A transaction's session id
        """
        return self.session.session_id

    @property
    def tx_id(self) -> Optional[str]:
        """
        Returns an id of open transaction or None otherwise

        :return: An id of open transaction or None otherwise
        """
        return self._tx_state.tx_id

    @property
    def last_query_stats(self):
        return self._last_query_stats

    def _tx_identity(self) -> _ydb_topic.TransactionIdentity:
        if not self.tx_id:
            raise RuntimeError("Unable to get tx identity without started tx.")
        return _ydb_topic.TransactionIdentity(self.tx_id, self.session_id)

    def _set_external_error(self, exc: BaseException) -> None:
        self._external_error = exc

    def _check_external_error_set(self):
        if self._external_error is None:
            return
        raise issues.ClientInternalError("Transaction was failed by external error.") from self._external_error

    def _begin_call(self, settings: Optional[BaseRequestSettings]) -> "BaseQueryTxContext":
        self._tx_state._check_invalid_transition(QueryTxStateEnum.BEGINED)

        return self._driver(
            _create_begin_transaction_request(self.session, self._tx_state),
            _apis.QueryService.Stub,
            _apis.QueryService.BeginTransaction,
            wrap_tx_begin_response,
            settings,
            (self.session, self._tx_state, self),
            preferred_endpoint=self.session._endpoint_key,
        )

    def _commit_call(self, settings: Optional[BaseRequestSettings]) -> "BaseQueryTxContext":
        self._check_external_error_set()
        self._tx_state._check_invalid_transition(QueryTxStateEnum.COMMITTED)

        return self._driver(
            _create_commit_transaction_request(self.session, self._tx_state),
            _apis.QueryService.Stub,
            _apis.QueryService.CommitTransaction,
            wrap_tx_commit_response,
            settings,
            (self.session, self._tx_state, self),
            preferred_endpoint=self.session._endpoint_key,
        )

    def _rollback_call(self, settings: Optional[BaseRequestSettings]) -> "BaseQueryTxContext":
        self._check_external_error_set()
        self._tx_state._check_invalid_transition(QueryTxStateEnum.ROLLBACKED)

        return self._driver(
            _create_rollback_transaction_request(self.session, self._tx_state),
            _apis.QueryService.Stub,
            _apis.QueryService.RollbackTransaction,
            wrap_tx_rollback_response,
            settings,
            (self.session, self._tx_state, self),
            preferred_endpoint=self.session._endpoint_key,
        )

    def _execute_call(
        self,
        query: str,
        parameters: Optional[dict],
        commit_tx: Optional[bool],
        syntax: Optional[base.QuerySyntax],
        exec_mode: Optional[base.QueryExecMode],
        stats_mode: Optional[base.QueryStatsMode],
        schema_inclusion_mode: Optional[base.QuerySchemaInclusionMode],
        result_set_format: Optional[base.QueryResultSetFormat],
        arrow_format_settings: Optional[base.ArrowFormatSettings],
        concurrent_result_sets: Optional[bool],
        settings: Optional[BaseRequestSettings],
    ) -> Iterable[_apis.ydb_query.ExecuteQueryResponsePart]:
        self._tx_state._check_tx_ready_to_use()
        self._check_external_error_set()

        self._last_query_stats = None

        request = base.create_execute_query_request(
            query=query,
            parameters=parameters,
            commit_tx=commit_tx,
            session_id=self.session.session_id,
            tx_id=self._tx_state.tx_id,
            tx_mode=self._tx_state.tx_mode,
            syntax=syntax,
            exec_mode=exec_mode,
            stats_mode=stats_mode,
            schema_inclusion_mode=schema_inclusion_mode,
            result_set_format=result_set_format,
            arrow_format_settings=arrow_format_settings,
            concurrent_result_sets=concurrent_result_sets,
        )

        return self._driver(
            request.to_proto(),
            _apis.QueryService.Stub,
            _apis.QueryService.ExecuteQuery,
            settings=settings,
            preferred_endpoint=self.session._endpoint_key,
        )

    def _move_to_beginned(self, tx_id: str) -> None:
        if self._tx_state._should_skip(QueryTxStateEnum.BEGINED) or not tx_id:
            return
        self._tx_state._change_state(QueryTxStateEnum.BEGINED)
        self._tx_state.tx_id = tx_id

    def _move_to_commited(self) -> None:
        if self._tx_state._should_skip(QueryTxStateEnum.COMMITTED):
            return
        self._tx_state._change_state(QueryTxStateEnum.COMMITTED)


class QueryTxContext(BaseQueryTxContext):
    def __init__(self, driver, session: "BaseQuerySession", tx_mode: base.BaseQueryTxMode):
        """
        An object that provides a simple transaction context manager that allows statements execution
        in a transaction. You don't have to open transaction explicitly, because context manager encapsulates
        transaction control logic, and opens new transaction if:

        1) By explicit .begin() method;
        2) On execution of a first statement, which is strictly recommended method, because that avoids useless round trip

        This context manager is not thread-safe, so you should not manipulate on it concurrently.

        :param driver: A driver instance
        :param session: A session instance
        :param tx_mode: Transaction mode, which is a one from the following choices:
         1) QuerySerializableReadWrite() which is default mode;
         2) QueryOnlineReadOnly(allow_inconsistent_reads=False);
         3) QuerySnapshotReadOnly();
         4) QuerySnapshotReadWrite();
         5) QueryStaleReadOnly().
        """

        super().__init__(driver, session, tx_mode)
        self._init_callback_handler(base.CallbackHandlerMode.SYNC)

    def __enter__(self) -> "BaseQueryTxContext":
        """
        Enters a context manager and returns a transaction

        :return: A transaction instance
        """
        return self

    def __exit__(self, *args, **kwargs):
        """
        Closes a transaction context manager and rollbacks transaction if
        it is not finished explicitly
        """
        self._ensure_prev_stream_finished()
        if self._tx_state._state == QueryTxStateEnum.BEGINED and self._external_error is None:
            # It's strictly recommended to close transactions directly
            # by using commit_tx=True flag while executing statement or by
            # .commit() or .rollback() methods, but here we trying to do best
            # effort to avoid useless open transactions
            logger.warning("Potentially leaked tx: %s", self._tx_state.tx_id)
            try:
                self.rollback()
            except issues.Error:
                logger.warning("Failed to rollback leaked tx: %s", self._tx_state.tx_id)

    def _ensure_prev_stream_finished(self) -> None:
        if self._prev_stream is not None:
            with self._prev_stream:
                pass
            self._prev_stream = None

    def begin(self, settings: Optional[BaseRequestSettings] = None) -> "QueryTxContext":
        """Explicitly begins a transaction

        :param settings: An additional request settings BaseRequestSettings;

        :return: Transaction object or exception if begin is failed
        """
        self._begin_call(settings)

        return self

    def commit(self, settings: Optional[BaseRequestSettings] = None) -> None:
        """Calls commit on a transaction if it is open otherwise is no-op. If transaction execution
        failed then this method raises PreconditionFailed.

        :param settings: An additional request settings BaseRequestSettings;

        :return: A committed transaction or exception if commit is failed
        """
        self._check_external_error_set()
        if self._tx_state._should_skip(QueryTxStateEnum.COMMITTED):
            return

        if self._tx_state._state == QueryTxStateEnum.NOT_INITIALIZED:
            self._tx_state._change_state(QueryTxStateEnum.COMMITTED)
            return

        self._ensure_prev_stream_finished()

        try:
            self._execute_callbacks_sync(base.TxEvent.BEFORE_COMMIT)
            self._commit_call(settings)
            self._execute_callbacks_sync(base.TxEvent.AFTER_COMMIT, exc=None)
        except BaseException as e:  # TODO: probably should be less wide
            self._execute_callbacks_sync(base.TxEvent.AFTER_COMMIT, exc=e)
            raise e

    def rollback(self, settings: Optional[BaseRequestSettings] = None) -> None:
        """Calls rollback on a transaction if it is open otherwise is no-op. If transaction execution
        failed then this method raises PreconditionFailed.

        :param settings: An additional request settings BaseRequestSettings;

        :return: A committed transaction or exception if commit is failed
        """
        self._check_external_error_set()
        if self._tx_state._should_skip(QueryTxStateEnum.ROLLBACKED):
            return

        if self._tx_state._state == QueryTxStateEnum.NOT_INITIALIZED:
            self._tx_state._change_state(QueryTxStateEnum.ROLLBACKED)
            return

        self._ensure_prev_stream_finished()

        try:
            self._execute_callbacks_sync(base.TxEvent.BEFORE_ROLLBACK)
            self._rollback_call(settings)
            self._execute_callbacks_sync(base.TxEvent.AFTER_ROLLBACK, exc=None)
        except BaseException as e:  # TODO: probably should be less wide
            self._execute_callbacks_sync(base.TxEvent.AFTER_ROLLBACK, exc=e)
            raise e

    def execute(
        self,
        query: str,
        parameters: Optional[dict] = None,
        commit_tx: Optional[bool] = False,
        syntax: Optional[base.QuerySyntax] = None,
        exec_mode: Optional[base.QueryExecMode] = None,
        concurrent_result_sets: Optional[bool] = False,
        settings: Optional[BaseRequestSettings] = None,
        *,
        stats_mode: Optional[base.QueryStatsMode] = None,
        schema_inclusion_mode: Optional[base.QuerySchemaInclusionMode] = None,
        result_set_format: Optional[base.QueryResultSetFormat] = None,
        arrow_format_settings: Optional[base.ArrowFormatSettings] = None,
    ) -> base.SyncResponseContextIterator:
        """Sends a query to Query Service

        :param query: (YQL or SQL text) to be executed.
        :param parameters: dict with parameters and YDB types;
        :param commit_tx: A special flag that allows transaction commit.
        :param syntax: Syntax of the query, which is a one from the following choices:
         1) QuerySyntax.YQL_V1, which is default;
         2) QuerySyntax.PG.
        :param exec_mode: Exec mode of the query, which is a one from the following choices:
         1) QueryExecMode.EXECUTE, which is default;
         2) QueryExecMode.EXPLAIN;
         3) QueryExecMode.VALIDATE;
         4) QueryExecMode.PARSE.
        :param concurrent_result_sets: A flag to allow YDB mix parts of different result sets. Default is False;
        :param settings: An additional request settings BaseRequestSettings;
        :param stats_mode: Mode of query statistics to gather, which is a one from the following choices:
         1) QueryStatsMode:NONE, which is default;
         2) QueryStatsMode.BASIC;
         3) QueryStatsMode.FULL;
         4) QueryStatsMode.PROFILE;
        :param schema_inclusion_mode: Schema inclusion mode for result sets:
         1) QuerySchemaInclusionMode.ALWAYS, which is default;
         2) QuerySchemaInclusionMode.FIRST_ONLY.
        :param result_set_format: Format of the result sets:
         1) QueryResultSetFormat.VALUE, which is default;
         2) QueryResultSetFormat.ARROW.
        :param arrow_format_settings: Settings for Arrow format when result_set_format is ARROW.

        :return: Iterator with result sets
        """
        self._ensure_prev_stream_finished()

        stream_it = self._execute_call(
            query=query,
            commit_tx=commit_tx,
            syntax=syntax,
            exec_mode=exec_mode,
            stats_mode=stats_mode,
            schema_inclusion_mode=schema_inclusion_mode,
            result_set_format=result_set_format,
            arrow_format_settings=arrow_format_settings,
            parameters=parameters,
            concurrent_result_sets=concurrent_result_sets,
            settings=settings,
        )

        self._prev_stream = base.SyncResponseContextIterator(
            stream_it,
            lambda resp: base.wrap_execute_query_response(
                rpc_state=None,
                response_pb=resp,
                session=self.session,
                tx=self,
                commit_tx=commit_tx,
                settings=self.session._settings,
            ),
            on_error=self.session._on_execute_stream_error,
        )
        return self._prev_stream



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class StatusIds(_message.Message):
    __slots__ = ()
    class StatusCode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        STATUS_CODE_UNSPECIFIED: _ClassVar[StatusIds.StatusCode]
        SUCCESS: _ClassVar[StatusIds.StatusCode]
        BAD_REQUEST: _ClassVar[StatusIds.StatusCode]
        UNAUTHORIZED: _ClassVar[StatusIds.StatusCode]
        INTERNAL_ERROR: _ClassVar[StatusIds.StatusCode]
        ABORTED: _ClassVar[StatusIds.StatusCode]
        UNAVAILABLE: _ClassVar[StatusIds.StatusCode]
        OVERLOADED: _ClassVar[StatusIds.StatusCode]
        SCHEME_ERROR: _ClassVar[StatusIds.StatusCode]
        GENERIC_ERROR: _ClassVar[StatusIds.StatusCode]
        TIMEOUT: _ClassVar[StatusIds.StatusCode]
        BAD_SESSION: _ClassVar[StatusIds.StatusCode]
        PRECONDITION_FAILED: _ClassVar[StatusIds.StatusCode]
        ALREADY_EXISTS: _ClassVar[StatusIds.StatusCode]
        NOT_FOUND: _ClassVar[StatusIds.StatusCode]
        SESSION_EXPIRED: _ClassVar[StatusIds.StatusCode]
        CANCELLED: _ClassVar[StatusIds.StatusCode]
        UNDETERMINED: _ClassVar[StatusIds.StatusCode]
        UNSUPPORTED: _ClassVar[StatusIds.StatusCode]
        SESSION_BUSY: _ClassVar[StatusIds.StatusCode]
        EXTERNAL_ERROR: _ClassVar[StatusIds.StatusCode]
    STATUS_CODE_UNSPECIFIED: StatusIds.StatusCode
    SUCCESS: StatusIds.StatusCode
    BAD_REQUEST: StatusIds.StatusCode
    UNAUTHORIZED: StatusIds.StatusCode
    INTERNAL_ERROR: StatusIds.StatusCode
    ABORTED: StatusIds.StatusCode
    UNAVAILABLE: StatusIds.StatusCode
    OVERLOADED: StatusIds.StatusCode
    SCHEME_ERROR: StatusIds.StatusCode
    GENERIC_ERROR: StatusIds.StatusCode
    TIMEOUT: StatusIds.StatusCode
    BAD_SESSION: StatusIds.StatusCode
    PRECONDITION_FAILED: StatusIds.StatusCode
    ALREADY_EXISTS: StatusIds.StatusCode
    NOT_FOUND: StatusIds.StatusCode
    SESSION_EXPIRED: StatusIds.StatusCode
    CANCELLED: StatusIds.StatusCode
    UNDETERMINED: StatusIds.StatusCode
    UNSUPPORTED: StatusIds.StatusCode
    SESSION_BUSY: StatusIds.StatusCode
    EXTERNAL_ERROR: StatusIds.StatusCode
    def __init__(self) -> None: ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from ydb._grpc.v6.draft.protos import ydb_maintenance_pb2 as draft_dot_protos_dot_ydb__maintenance__pb2

GRPC_GENERATED_VERSION = '1.76.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + ' but the generated code in draft/ydb_maintenance_v1_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class MaintenanceServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListClusterNodes = channel.unary_unary(
                '/Ydb.Maintenance.V1.MaintenanceService/ListClusterNodes',
                request_serializer=draft_dot_protos_dot_ydb__maintenance__pb2.ListClusterNodesRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__maintenance__pb2.ListClusterNodesResponse.FromString,
                _registered_method=True)
        self.CreateMaintenanceTask = channel.unary_unary(
                '/Ydb.Maintenance.V1.MaintenanceService/CreateMaintenanceTask',
                request_serializer=draft_dot_protos_dot_ydb__maintenance__pb2.CreateMaintenanceTaskRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__maintenance__pb2.MaintenanceTaskResponse.FromString,
                _registered_method=True)
        self.RefreshMaintenanceTask = channel.unary_unary(
                '/Ydb.Maintenance.V1.MaintenanceService/RefreshMaintenanceTask',
                request_serializer=draft_dot_protos_dot_ydb__maintenance__pb2.RefreshMaintenanceTaskRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__maintenance__pb2.MaintenanceTaskResponse.FromString,
                _registered_method=True)
        self.GetMaintenanceTask = channel.unary_unary(
                '/Ydb.Maintenance.V1.MaintenanceService/GetMaintenanceTask',
                request_serializer=draft_dot_protos_dot_ydb__maintenance__pb2.GetMaintenanceTaskRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__maintenance__pb2.GetMaintenanceTaskResponse.FromString,
                _registered_method=True)
        self.ListMaintenanceTasks = channel.unary_unary(
                '/Ydb.Maintenance.V1.MaintenanceService/ListMaintenanceTasks',
                request_serializer=draft_dot_protos_dot_ydb__maintenance__pb2.ListMaintenanceTasksRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__maintenance__pb2.ListMaintenanceTasksResponse.FromString,
                _registered_method=True)
        self.DropMaintenanceTask = channel.unary_unary(
                '/Ydb.Maintenance.V1.MaintenanceService/DropMaintenanceTask',
                request_serializer=draft_dot_protos_dot_ydb__maintenance__pb2.DropMaintenanceTaskRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__maintenance__pb2.ManageMaintenanceTaskResponse.FromString,
                _registered_method=True)
        self.CompleteAction = channel.unary_unary(
                '/Ydb.Maintenance.V1.MaintenanceService/CompleteAction',
                request_serializer=draft_dot_protos_dot_ydb__maintenance__pb2.CompleteActionRequest.SerializeToString,
                response_deserializer=draft_dot_protos_dot_ydb__maintenance__pb2.ManageActionResponse.FromString,
                _registered_method=True)


class MaintenanceServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ListClusterNodes(self, request, context):
        """List cluster nodes.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateMaintenanceTask(self, request, context):
        """Create maintenance task.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RefreshMaintenanceTask(self, request, context):
        """Try to perform maintenance task's actions (polling).
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetMaintenanceTask(self, request, context):
        """Get detailed task information.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListMaintenanceTasks(self, request, context):
        """List maintenance tasks.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DropMaintenanceTask(self, request, context):
        """Drop maintenance task.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CompleteAction(self, request, context):
        """Mark action as completed.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MaintenanceServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ListClusterNodes': grpc.unary_unary_rpc_method_handler(
                    servicer.ListClusterNodes,
                    request_deserializer=draft_dot_protos_dot_ydb__maintenance__pb2.ListClusterNodesRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__maintenance__pb2.ListClusterNodesResponse.SerializeToString,
            ),
            'CreateMaintenanceTask': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateMaintenanceTask,
                    request_deserializer=draft_dot_protos_dot_ydb__maintenance__pb2.CreateMaintenanceTaskRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__maintenance__pb2.MaintenanceTaskResponse.SerializeToString,
            ),
            'RefreshMaintenanceTask': grpc.unary_unary_rpc_method_handler(
                    servicer.RefreshMaintenanceTask,
                    request_deserializer=draft_dot_protos_dot_ydb__maintenance__pb2.RefreshMaintenanceTaskRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__maintenance__pb2.MaintenanceTaskResponse.SerializeToString,
            ),
            'GetMaintenanceTask': grpc.unary_unary_rpc_method_handler(
                    servicer.GetMaintenanceTask,
                    request_deserializer=draft_dot_protos_dot_ydb__maintenance__pb2.GetMaintenanceTaskRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__maintenance__pb2.GetMaintenanceTaskResponse.SerializeToString,
            ),
            'ListMaintenanceTasks': grpc.unary_unary_rpc_method_handler(
                    servicer.ListMaintenanceTasks,
                    request_deserializer=draft_dot_protos_dot_ydb__maintenance__pb2.ListMaintenanceTasksRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__maintenance__pb2.ListMaintenanceTasksResponse.SerializeToString,
            ),
            'DropMaintenanceTask': grpc.unary_unary_rpc_method_handler(
                    servicer.DropMaintenanceTask,
                    request_deserializer=draft_dot_protos_dot_ydb__maintenance__pb2.DropMaintenanceTaskRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__maintenance__pb2.ManageMaintenanceTaskResponse.SerializeToString,
            ),
            'CompleteAction': grpc.unary_unary_rpc_method_handler(
                    servicer.CompleteAction,
                    request_deserializer=draft_dot_protos_dot_ydb__maintenance__pb2.CompleteActionRequest.FromString,
                    response_serializer=draft_dot_protos_dot_ydb__maintenance__pb2.ManageActionResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'Ydb.Maintenance.V1.MaintenanceService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('Ydb.Maintenance.V1.MaintenanceService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class MaintenanceService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ListClusterNodes(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.Maintenance.V1.MaintenanceService/ListClusterNodes',
            draft_dot_protos_dot_ydb__maintenance__pb2.ListClusterNodesRequest.SerializeToString,
            draft_dot_protos_dot_ydb__maintenance__pb2.ListClusterNodesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CreateMaintenanceTask(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.Maintenance.V1.MaintenanceService/CreateMaintenanceTask',
            draft_dot_protos_dot_ydb__maintenance__pb2.CreateMaintenanceTaskRequest.SerializeToString,
            draft_dot_protos_dot_ydb__maintenance__pb2.MaintenanceTaskResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def RefreshMaintenanceTask(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.Maintenance.V1.MaintenanceService/RefreshMaintenanceTask',
            draft_dot_protos_dot_ydb__maintenance__pb2.RefreshMaintenanceTaskRequest.SerializeToString,
            draft_dot_protos_dot_ydb__maintenance__pb2.MaintenanceTaskResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetMaintenanceTask(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.Maintenance.V1.MaintenanceService/GetMaintenanceTask',
            draft_dot_protos_dot_ydb__maintenance__pb2.GetMaintenanceTaskRequest.SerializeToString,
            draft_dot_protos_dot_ydb__maintenance__pb2.GetMaintenanceTaskResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListMaintenanceTasks(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.Maintenance.V1.MaintenanceService/ListMaintenanceTasks',
            draft_dot_protos_dot_ydb__maintenance__pb2.ListMaintenanceTasksRequest.SerializeToString,
            draft_dot_protos_dot_ydb__maintenance__pb2.ListMaintenanceTasksResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DropMaintenanceTask(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.Maintenance.V1.MaintenanceService/DropMaintenanceTask',
            draft_dot_protos_dot_ydb__maintenance__pb2.DropMaintenanceTaskRequest.SerializeToString,
            draft_dot_protos_dot_ydb__maintenance__pb2.ManageMaintenanceTaskResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CompleteAction(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/Ydb.Maintenance.V1.MaintenanceService/CompleteAction',
            draft_dot_protos_dot_ydb__maintenance__pb2.CompleteActionRequest.SerializeToString,
            draft_dot_protos_dot_ydb__maintenance__pb2.ManageActionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)
